@@ -15,6 +15,7 @@ from pytz import _UTC, timezone
 
 from ..utils.changelog import ChangeLog, Entry
 from ..utils.conf import get_config_path, get_credentials
+from ..utils.mapping import Mappable
 
 sys.path.append(os.path.dirname(__file__))
 
@@ -77,6 +78,19 @@ Tue Dec  3 10:13:08 UTC 2019 - Andreas Hasenkopf <ahasenkopf@suse.com>
 """
 
 
+class TestMappable(TestCase):
+    def test(self):
+        m = Mappable(a="a", b="b")
+        m["c"] = "c"
+
+        for key in ('a', 'b', 'c'):
+            with self.subTest(f"get {key}"):
+                self.assertEqual(key, m.get(key))
+
+        with self.subTest("Default"):
+            self.assertEqual("føø", m.get("d", "føø"))
+
+
 class TestEntry(TestCase):
     def test_timestamp(self):
         cet = timezone("Europe/Berlin")
@@ -317,7 +331,7 @@ class TestChangeLog(TestCase):
         self.assertIn("Cannot parse changelog entry", wmock.call_args[0][0])
 
 
-@mock.patch("osc.conf", side_effect=ImportError, create=True)
+@mock.patch("osctiny.utils.conf._conf", new_callable=lambda: None, create=True)
 @mock.patch("pathlib.Path.is_file", return_value=True)
 class TestConfig(TestCase):
     def test_get_config_path(self, *_):
@@ -334,8 +348,8 @@ class TestConfig(TestCase):
         _, path1 = mkstemp()
         _, path2 = mkstemp()
 
-        expected_insecure_credentials = ("my-dummy-user", "my-insecure-dummy-password")
-        expected_secure_credentials = ('my-dummy-user', 'my-secure-dummy-password')
+        expected_insecure_credentials = ("my-dummy-user", "my-insecure-dummy-password", None)
+        expected_secure_credentials = ('my-dummy-user', 'my-secure-dummy-password', None)
 
         with open(path1, "w") as handle:
             handle.write("[http://api.dummy-bs.org]\n")
